#!/usr/bin/env python
# encoding: utf-8
"""
twitterbot.py

lightweight Python glue between twitter and growl

Features:
stores tweets while you are away - displaying them when you get back to your screen
lets you designate a list of "stickers" - people you follow whose tweets you want to stay on screen till dismissed
caches profile images

requires:
http://mike.verdone.ca/twitter/
http://growl.info/downloads_developers.php

Created by Preston Holmes on 2009-02-12. preston@ptone.com

Copyright (c) 2009 Preston Holmes

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
"""

import sys
import os
from twitter.api import Twitter, TwitterError
from collections import deque
import time
import Growl
from subprocess import Popen, PIPE
import getpass
import urllib2
import hashlib
import socket

def user_is_active(secs_idle=12):
    cmd = '''echo $((`ioreg -c IOHIDSystem | sed -e '/HIDIdleTime/ !{ d' -e 't' -e '}' -e 's/.* = //g' -e 'q'` / 1000000000))'''
    time_since_active = int(Popen(cmd,shell=True,stdout=PIPE,stderr=PIPE).communicate()[0])
    if time_since_active < secs_idle:
        return True
    else:
        return False
               
def get_growl_image(url):
    path_to_folder  = os.path.dirname(os.path.realpath(__file__))
    cache_folder = os.path.join(path_to_folder,'profile image cache')
    if not os.path.exists(cache_folder):
        os.makedirs(cache_folder)
    fname = '%s.%s' % (hashlib.md5(url).hexdigest(), url.split('.')[-1])
    cached_image = os.path.join(cache_folder,fname)
    image = None
    if os.path.exists(cached_image):
        mtime = os.path.getmtime(cached_image)
        #invalidate if over 3 days old
        if (time.time() - mtime) > (60 * 60 * 24 * 3):
            os.remove(cached_image)
        else:
            image = Growl.Image.imageFromPath(cached_image)
    if not image:
        f = open(cached_image,'wb')
        f.write(urllib2.urlopen(url).read())
        f.close()
        image = Growl.Image.imageFromPath(cached_image)
    return image        
        
def main():
    #Change these settings if you want
    #Twitter username, if None will prompt
    user = None
    
    #Twitter Password, if None will prompt
    pw = None
    
    #whether all statuses should stick (require click to dismiss)
    to_stick = False
    
    #list of screen names of people who should always stick
    #ie ('gruber','JohnCleese','hodgman')
    stickers = ()
    
    #display name - should either be "screen_name" or "name"
    display_name = 'name'
    
    #max number of status to display when you return from inactivity:
    max_display = 10
    
    #seconds of idle to be considered away:
    secs_idle = 12
    
    # api_frequence - number of seconds between checking with twitter.  Max 100 per hour across all your clients
    # so if you use iPhone or other apps a lot, be sure to leave yourself some extra checks
    api_delay = 100
    
    #delay between notifications
    #if there is more than one tweet in the queue, how long to pause between growl notifications
    #this will depend somewhat on your growl settings
    notify_delay = 3
    #-----------------------------------------end settings------------------------------------------------------
    
    status_queue = deque()
    lastid = None
    last_api_call = 0
    
    if not user:
        user = raw_input('Twitter Account Name: ')
    if not pw:
        pw = getpass.unix_getpass("Enter your password:")
        
    twitter = Twitter(user,pw)
    notifier = Growl.GrowlNotifier('twitter_monitor', ['status'])
    notifier.register()
    
    while True:
        #wait api_delay seconds between API calls 100 max per hour
        try:
            if (int(time.time()) - last_api_call) > api_delay:
                if lastid:
                    response = twitter.statuses.friends_timeline(since_id=lastid)
                else:
                    response = twitter.statuses.friends_timeline(count=5)
            
                if response:
                    lastid = response[0]['id']
                
                response.reverse()
                status_queue.extend(response)
                last_api_call = time.time()
                #print lastid
        
        
            if status_queue:
                if user_is_active():
                    ct = len(status_queue)
                    #print ct
                    if ct > max_display:
                        notifier.notify('status','Too many tweets',
                            'Check out your latest tweets on twitter.com',sticky=True)
                        for i in range(0,ct):
                            status = status_queue.popleft()
                            print "@%s: %s\n" % (status['user'][display_name],status['text'])
                        status_queue.clear()
                    else:
                        for i in range(0,ct):
                            status = status_queue.popleft()
                            image = get_growl_image(status['user']['profile_image_url'])
                            this_stick = to_stick or (status['user']['screen_name'] in stickers)
                            notifier.notify('status',status['user'][display_name],status['text'],icon=image,sticky=this_stick)
                            print "@%s: %s\n" % (status['user'][display_name],status['text'])
                            time.sleep(notify_delay)
        except socket.gaierror, e:
            notifier.notify('status','Network Error',"Unable to reach the server")
            time.sleep(20)
        except TwitterError, e:
            import simplejson as json
            error_s = str(e.args[0]).split(':', 1)[1]
            #print error_s
            error_dict = json.loads(error_s)
            notifier.notify('status','Twitter Error',error_dict['error'])
            sys.exit(1)
        
        except:
            print sys.exc_info()[0]
            raise
        time.sleep(5)

if __name__ == '__main__':
    main()

